@@ -10,14 +10,61 @@ from typing import Dict
 import mutagen
 import pandas as pd
 from joblib import Parallel, delayed
-from rich import inspect
 from tinytag import TinyTag
 
-from .db import fetchall_dict, sqlite_con
-from .subtitle import get_subtitle, is_file_with_subtitle, youtube_dl_id
-from .utils import chunks, cmd, combine, get_video_files, log, remove_None, safe_unpack
+from xklb.db import fetchall_dict, sqlite_con
+from xklb.subtitle import get_subtitle, is_file_with_subtitle, youtube_dl_id
+from xklb.utils import (
+    SQLITE_PARAM_LIMIT,
+    chunks,
+    cmd,
+    combine,
+    get_media_files,
+    log,
+    remove_media,
+    remove_None,
+    safe_unpack,
+)
 
-SQLITE_PARAM_LIMIT = 32765
+audio_include_string = (
+    lambda x: f"""and (
+    filename like :include{x}
+    OR mood like :include{x}
+    OR genre like :include{x}
+    OR year like :include{x}
+    OR bpm like :include{x}
+    OR key like :include{x}
+    OR time like :include{x}
+    OR decade like :include{x}
+    OR categories like :include{x}
+    OR city like :include{x}
+    OR country like :include{x}
+    OR description like :include{x}
+    OR album like :include{x}
+    OR title like :include{x}
+    OR artist like :include{x}
+)"""
+)
+
+audio_exclude_string = (
+    lambda x: f"""and (
+    filename not like :exclude{x}
+    OR mood not like :exclude{x}
+    OR genre not like :exclude{x}
+    OR year not like :exclude{x}
+    OR bpm not like :exclude{x}
+    OR key not like :exclude{x}
+    OR time not like :exclude{x}
+    OR decade not like :exclude{x}
+    OR categories not like :exclude{x}
+    OR city not like :exclude{x}
+    OR country not like :exclude{x}
+    OR description not like :exclude{x}
+    OR album not like :exclude{x}
+    OR title not like :exclude{x}
+    OR artist not like :exclude{x}
+)"""
+)
 
 
 def get_provenance(file):
@@ -82,17 +129,7 @@ def parse_tags(mutagen: Dict, tinytag: Dict):
 def extract_metadata(args, f):
     try:
         probe = json.loads(
-            cmd(
-                "ffprobe",
-                "-loglevel",
-                "quiet",
-                "-print_format",
-                "json=compact=1",
-                "-show_entries",
-                "format",
-                f,
-                quiet=True,
-            ).stdout
+            cmd("ffprobe", "-loglevel", "quiet", "-print_format", "json=compact=1", "-show_entries", "format", f).stdout
         )
     except (KeyboardInterrupt, SystemExit):
         exit(130)
@@ -166,13 +203,13 @@ def optimize_db(args):
     if Path(args.db).exists():
         cmd("sqlite-utils", "optimize", args.db)
         columns = cmd(
-            "sqlite-utils", "tables", args.db, "--columns", "|", "jq", "-r", "'.[0].columns[]'", quiet=True
+            f"sqlite-utils tables {args.db} --columns | jq -r '.[0].columns[]'", quiet=True, shell=True
         ).stdout.splitlines()
         for column in columns:
             cmd("sqlite-utils", "create-index", "--if-not-exists", "--analyze", args.db, "media", column)
 
 
-def extract_chunk(args, con, l):
+def extract_chunk(args, l):
     metadata = (
         Parallel(n_jobs=-1 if args.verbose == 0 else 1, backend="threading")(
             delayed(extract_metadata)(args, file) for file in l
@@ -186,7 +223,7 @@ def extract_chunk(args, con, l):
             DF.year = DF.year.astype(str)
     DF.apply(pd.to_numeric, errors="ignore").convert_dtypes().to_sql(  # type: ignore
         "media",
-        con=con,
+        con=args.con,
         if_exists="append",
         index=False,
         chunksize=70,
@@ -194,15 +231,15 @@ def extract_chunk(args, con, l):
     )
 
 
-def find_new_files(args, con, path):
-    video_files = get_video_files(path, args.audio)
+def find_new_files(args, path):
+    video_files = get_media_files(path, args.audio)
     new_files = set(video_files)
 
     try:
         existing = set(
             map(
                 lambda x: x["filename"],
-                fetchall_dict(con, f"select filename from media where filename like '{path}%'"),
+                fetchall_dict(args.con, f"select filename from media where filename like '{path}%'"),
             )
         )
     except:
@@ -211,28 +248,19 @@ def find_new_files(args, con, path):
         video_files = list(new_files - existing)
 
         deleted_files = list(existing - new_files)
-        if len(deleted_files) > 0:
-            print(f"Removing {len(deleted_files)} orphaned metadata")
+        remove_media(args, deleted_files)
 
-            df_chunked = chunks(deleted_files, SQLITE_PARAM_LIMIT)
-            for l in df_chunked:
-                con.execute(
-                    "delete from media where filename in (" + ",".join(["?"] * len(l)) + ")",
-                    (*l,),
-                )
-                con.commit()
-
-        con.execute("delete from media where filename like '%/keep/%'")
-        con.commit()
+        args.con.execute("DELETE from media where filename like '%/keep/%'")
+        args.con.commit()
 
     return video_files
 
 
-def scan_path(args, con, path):
+def scan_path(args, path):
     path = Path(path).resolve()
     print(f"{path} : Scanning...")
 
-    video_files = find_new_files(args, con, path)
+    video_files = find_new_files(args, path)
 
     if len(video_files) > 0:
         print(f"Adding {len(video_files)} new media")
@@ -244,16 +272,25 @@ def scan_path(args, con, path):
         for idx, l in enumerate(df_chunked):
             percent = ((batch_count * idx) + len(l)) / len(video_files) * 100
             print(f"Extracting metadata: {percent:3.1f}% (chunk {idx + 1} of {chunks_count})")
-            extract_chunk(args, con, l)
+            extract_chunk(args, l)
 
             if args.subtitle:
                 print("Fetching subtitles")
                 Parallel(n_jobs=5)(delayed(get_subtitle)(args, file) for file in l)
 
 
+def extractor(args):
+    Path(args.db).touch()
+    args.con = sqlite_con(args.db)
+    for path in args.paths:
+        scan_path(args, path)
+
+    optimize_db(args)
+
+
 def main():
     parser = argparse.ArgumentParser()
-    parser.add_argument("db")
+    parser.add_argument("db", nargs="?")
     parser.add_argument("paths", nargs="*")
     parser.add_argument("-a", "--audio", action="store_true")
     parser.add_argument("-s", "--subtitle", action="store_true")
@@ -262,14 +299,17 @@ def main():
     parser.add_argument("-f", "--force-rescan", action="store_true")
     parser.add_argument("-v", "--verbose", action="count", default=0)
     args = parser.parse_args()
+
+    if not args.db:
+        if args.audio:
+            args.db = "audio.db"
+        else:
+            args.db = "video.db"
+
     if args.force_rescan:
         Path(args.db).unlink(missing_ok=True)
 
-    con = sqlite_con(args.db)
-    for path in args.paths:
-        scan_path(args, con, path)
-
-    optimize_db(args)
+    extractor(args)
 
 
 if __name__ == "__main__":
